System.register("utils", [], function (exports_1, context_1) {
    "use strict";
    var __moduleName = context_1 && context_1.id;
    /**
     * @private
     */
    function flatten(object, separator = '.') {
        const isValidObject = (value) => {
            if (!value) {
                return false;
            }
            const isArray = Array.isArray(value);
            const isBuffer = Buffer.isBuffer(value);
            const isΟbject = Object.prototype.toString.call(value) === '[object Object]';
            const hasKeys = !!Object.keys(value).length;
            return !isArray && !isBuffer && isΟbject && hasKeys;
        };
        const walker = (child, path = []) => {
            return Object.assign({}, ...Object.keys(child).map((key) => isValidObject(child[key])
                ? walker(child[key], path.concat([key]))
                : { [path.concat([key]).join(separator)]: child[key] }));
        };
        return Object.assign({}, walker(object));
    }
    exports_1("flatten", flatten);
    /**
     * @private
     */
    function mergeRoles(dst, ...srcs) {
        for (const src of srcs) {
            for (const [key, val] of Object.entries(src)) {
                if (typeof val === 'object') {
                    dst[key] = Object.assign({}, dst[key], val);
                }
                else {
                    dst[key] = val;
                }
            }
        }
        return dst;
    }
    exports_1("mergeRoles", mergeRoles);
    return {
        setters: [],
        execute: function () {
        }
    };
});
System.register("index", ["utils"], function (exports_2, context_2) {
    "use strict";
    var utils_1, SEPARATOR, RBAC;
    var __moduleName = context_2 && context_2.id;
    return {
        setters: [
            function (utils_1_1) {
                utils_1 = utils_1_1;
            }
        ],
        execute: function () {
            SEPARATOR = ':';
            /**
             * RBAC classref
             */
            RBAC = class RBAC {
                /**
                 * RBAC constructor
                 * @param options RBAC options
                 */
                constructor(options = {}) {
                    /**
                     * RBAC roles object
                     */
                    this._rules = {};
                    this._rulesCompiled = {};
                    this._refs = {};
                    this._memoize = true;
                    const { roles = {}, memoize = true } = options;
                    this._memoize = memoize;
                    for (const [roleName, permissions] of Object.entries(roles)) {
                        if (permissions.can.length !== 0) {
                            this._rules[roleName] = this._rules[roleName] || {};
                            const resRule = this._rules[roleName];
                            for (const permission of permissions.can) {
                                if (typeof permission === 'string') {
                                    const [resource, operation = '*'] = permission.split(SEPARATOR);
                                    resRule[resource] = resRule[resource] || {};
                                    const opRule = resRule[resource];
                                    opRule[operation] = true;
                                }
                                else {
                                    const [resource, operation] = permission.operation
                                        ? [permission.name, permission.operation]
                                        : permission.name.split(SEPARATOR);
                                    resRule[resource] = resRule[resource] || {};
                                    const opRule = resRule[resource];
                                    opRule[operation] =
                                        typeof permission.when !== 'undefined' ? permission.when : true;
                                }
                            }
                        }
                        if (permissions.inherits) {
                            for (const refRole of permissions.inherits) {
                                this._refs[roleName] = this._refs[roleName] || {};
                                // this._rules[refRole] = this._rules[refRole] || {};
                                this._refs[roleName][refRole] = this._rules[refRole];
                            }
                        }
                    }
                    this._compile();
                }
                _collectRefs(refs) {
                    if (typeof refs === 'undefined') {
                        return {};
                    }
                    const refsToCompile = {};
                    for (const refRole of Object.keys(refs)) {
                        utils_1.mergeRoles(refsToCompile, this._collectRefs(this._refs[refRole]), refs[refRole]);
                    }
                    return refsToCompile;
                }
                _compile() {
                    const refsToCompile = {};
                    for (const [role, ref] of Object.entries(this._refs)) {
                        refsToCompile[role] = refsToCompile[role] || {};
                        refsToCompile[role] = this._collectRefs(ref);
                    }
                    this._rulesCompiled = Object.assign({}, utils_1.flatten(refsToCompile, SEPARATOR), utils_1.flatten(this._rules, SEPARATOR));
                }
                /**
                 * Adds new role to rules.
                 * @public
                 * @version 1.1.X
                 * @param role user role
                 * @param resource resource to access
                 * @param operation allowed operation
                 * @param when function for additional checks
                 */
                add(role, resource, operation, when) {
                    this._rules[role] = this._rules[role] || {};
                    this._rules[role][resource] = this._rules[role][resource] || {};
                    this._rules[role][resource][operation] =
                        typeof when === 'undefined' ? true : when;
                    this._compile();
                }
                /**
                 * Remove rule(s).
                 * @public
                 * @version 1.1.X
                 * @param role user role
                 * @param resource resource to access
                 * @param operation operation
                 */
                remove(role, resource = '*', operation = '*') {
                    if (resource === '*') {
                        if (operation === '*') {
                            // role:*:*
                            delete this._rules[role];
                        }
                        else {
                            // role:*:operation
                            if (this._rules[role][resource]) {
                                delete this._rules[role][resource][operation];
                            }
                            for (const [res, opRule] of Object.entries(this._rules[role])) {
                                if (Object.keys(opRule).indexOf(operation) !== -1) {
                                    delete this._rules[role][res][operation];
                                }
                            }
                        }
                    }
                    else {
                        if (operation === '*') {
                            // role:resource:*
                            delete this._rules[role][resource];
                        }
                        else {
                            // role:resource:operation
                            delete this._rules[role][resource][operation];
                        }
                    }
                    this._compile();
                }
                can(role, resource, operation, context) {
                    let check = false;
                    const checkWhen = typeof context !== 'undefined';
                    const rule = [role, resource, operation].join(SEPARATOR);
                    if (this._rulesCompiled.hasOwnProperty(rule)) {
                        check = this._rulesCompiled[rule];
                    }
                    else {
                        const wildcardRules = [
                            [role, resource, '*'].join(SEPARATOR),
                            [role, '*', operation].join(SEPARATOR),
                            [role, '*', '*'].join(SEPARATOR),
                        ];
                        for (const wcr of wildcardRules) {
                            if (this._rulesCompiled.hasOwnProperty(wcr)) {
                                check = this._rulesCompiled[wcr];
                                if (this._memoize) {
                                    // add to rules
                                    this._rulesCompiled[rule] = this._rulesCompiled[wcr];
                                    this._rules[role][resource] = this._rules[role][resource] || {};
                                    this._rules[role][resource][operation] = check;
                                }
                            }
                        }
                    }
                    if (typeof check === 'boolean') {
                        return check;
                    }
                    if (checkWhen) {
                        return check(context);
                    }
                    return Boolean(check);
                }
            };
            exports_2("RBAC", RBAC);
            exports_2("default", RBAC);
        }
    };
});
//# sourceMappingURL=rbac.bundle.system.js.map